
		/***********************************************/
		/*   Copyright(c)  1994,  W.M.  Brown, Dallas  */
		/*   Texas.				       */
		/*   					       */
		/*   This   file   is  part  of  Ghostscript.  */
		/*   Ghostscript  is  distributed in the hope  */
		/*   that  it  will  be  useful,  but WITHOUT  */
		/*   ANY  WARRANTY.  No author or distributor  */
		/*   accepts  responsibility  to  anyone  for  */
		/*   the  consequences  of  using  it  or for  */
		/*   whether   it   serves   any   particular  */
		/*   purpose  or  works  at  all,  unless  he  */
		/*   says   so   in  writing.  Refer  to  the  */
		/*   Ghostscript  General  Public License for  */
		/*   full details.			       */
		/*   					       */
		/*   Everyone   is   granted   permission  to  */
		/*   copy,     modify     and    redistribute  */
		/*   Ghostscript,    but   only   under   the  */
		/*   conditions      described     in     the  */
		/*   Ghostscript  General  Public  License. A  */
		/*   copy  of  this  license  is  supposed to  */
		/*   have   been  given  to  you  along  with  */
		/*   Ghostscript   so   you   can  know  your  */
		/*   rights  and  responsibilities. It should  */
		/*   be   in  a  file  named  COPYING.  Among  */
		/*   other  things,  the copyright notice and  */
		/*   this  notice  must  be  preserved on all  */
		/*   copies.				       */
		/***********************************************/

		/***********************************************/
   	/*   gdevphoton.c   QNX   Photon  driver  for  */
   	/*   		Ghostscript	                 	        */
   	/*	Author:  Bill  Brown		                    */
   	/*   WMB Consultants, Dallas, TX October 1994  */
   	/***********************************************/
		/*   Revised April 1995, New beta (wmb)  		  */

/*********************************************************************
*   IN  NO  EVENT SHALL WILLIAM M. BROWN BE LIABLE FOR ANY SPECIAL,  *
*   INCIDENTAL,  INDIRECT  OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR  *
*   ANY  DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE, DATA OR  *
*   PROFITS,  WHETHER  OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE,  *
*   AND   ON  ANY  THEORY  OF  LIABILITY,  ARISING  OUT  OF  OR  IN  *
*   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.	     *		*
*********************************************************************/


#include "gx.h"
#include "gxdevice.h"
#include <Pt.h>


/********************************************************************/


typedef struct gx_device_photon_s {
	gx_device_common;
	byte *fbaddr;
	unsigned regs;
} gx_device_photon;

#define photondev ((gx_device_photon *)dev)

/* These are adjusted later */                    		

#define XSIZE 536 			
#define YSIZE 693 			
#define XPPI  63.0			
#define YPPI  63.0			

dev_proc_open_device(photon_open);
dev_proc_close_device(photon_close);
dev_proc_fill_rectangle(photon_fill_rectangle);
dev_proc_copy_mono(photon_copy_mono);
dev_proc_copy_color(photon_copy_color); 
dev_proc_sync_output(photon_sync_output);
dev_proc_map_rgb_color(photon_map_rgb_color);
dev_proc_map_color_rgb(photon_map_color_rgb); 
dev_proc_output_page(photon_output_page);

static gx_device_procs photon_procs =
{	photon_open,
	gx_default_get_initial_matrix,
	photon_sync_output,
	photon_output_page,
	photon_close,
	photon_map_rgb_color,
   photon_map_color_rgb, 
	photon_fill_rectangle,
	gx_default_tile_rectangle,
	photon_copy_mono,
	photon_copy_color,
	gx_default_draw_line,
	gx_default_get_bits,
	gx_default_get_props,
	gx_default_put_props
};

gx_device_photon gs_photon_device = 
{	sizeof(gx_device_photon),
	&photon_procs,
	"photon",
	XSIZE, YSIZE,
	XPPI, YPPI,
	no_margins,
	dci_color(16,32767,4),	       /*	dci_color(8,255,4) */
	0
};

PhGC_t		*gc, *old_gc;
static int	isOpen = 0;

int 
photon_open(gx_device *dev)
{		 
	PhPoint_t	point;
	PtArg_t		arg[3];
	PtWidget_t	*window;
    PhRegion_t  info;

	if(isOpen == 1)
		return(0);
			
	if( PtInit( NULL)) {
		fprintf(stderr,"\n\nPhoton is not running, Please select another device.\n");
		return(-1);
		}  
	PhEventArm();
	point.x = dev->width;
	point.y = dev->height;
	
	PtSetArg(&arg[0],Pt_ARG_DIM,&point,0);
	PtSetArg(&arg[1],Pt_ARG_WINDOW_FORCE_FRONT,Pt_TRUE,0);
	window = PtCreateWidget(PtWindow,NULL,2,arg); 
	PtRealizeWidget(window); 

    memset(&info,0,sizeof(info));			
    info.events_sense = 0;
    info.events_opaque = 0xffffFFFF;
    info.rid = window->rid;
    PhRegionChange(Ph_REGION_EV_SENSE,0,&info,NULL,NULL);
	gc     = PgCreateGC( 0 );
	old_gc = PgSetGC( gc );
	
	PgSetRegion( window->rid );
	/***********************************************/
	/*   Some  PS  files open the device twice so  */
	/*   to  preven  this  we  flag the fact that  */
	/*   we are open										  */
	/***********************************************/
	isOpen = 1;		
	return 0; /*success */	

}

int 
photon_close(gx_device *dev)
{  	
	/* restore GC */
	PgSetGC( old_gc );
	
	return 0;
}

int 
photon_fill_rectangle(gx_device *dev, int x1, int y1, int w, int h,
                      gx_color_index color)
{
	
	fit_fill(dev, x1, y1, w, h);
	
	PgSetFillColor(color | Pg_MIX_COLOR);
	PgSetStrokeColor(color | Pg_MIX_COLOR);
	PgDrawIRect(x1,y1,x1+w,y1+h,Pg_DRAW_FILL_STROKE);

	return 0;
}

int 
photon_copy_mono(gx_device *dev,
		 byte *base, int sourcex, int raster, gx_bitmap_id id,
       int x, int y, int w, int h, 
		 gx_color_index zero, gx_color_index one)
{				
	PhPoint_t	pos,area;
	int i; 
	int nbytes = (h * raster);

	fit_copy(dev, base, sourcex, raster, id, x, y, w, h);

	pos.x = x;
	pos.y = y;
	
	area.x = w;
	area.y = h;
	
	if (one != gx_no_color_index) 
	{	
		PgSetTextColor(one);
		PgDrawBitmap(base,0,&pos,&area,raster,0);
	}

	if (zero != gx_no_color_index)
	{
		for (i = 0; i < nbytes; i++) base[i] = ~base[i];
		PgSetTextColor(zero);
		PgDrawBitmap(base,0,&pos,&area,raster,0);
	}

	return 0;

}				
/***************************INCOMPLETE !!!!****************************/
/*   Copy  a  color  pixelmap.  This  is  just like a bitmap, except  */
/*   that  each  pixel  takes 8 bits instead of 1.                    */
/*   I have yet to find a situation that calls this (wmb) 				 */
/**********************************************************************/
int
photon_copy_color(gx_device *dev,
  byte *base, int sourcex, int raster, gx_bitmap_id id,
  int x, int y, int w, int h)
{	
	PhPoint_t	pos,area;
	int i; 

	fit_copy(dev, base, sourcex, raster, id, x, y, w, h);

	pos.x = x;
	pos.y = y;
	
	area.x = w;
	area.y = h;
	
	PgDrawBitmap(base,0,&pos,&area,raster,0);
	
	return 0;
}

int
photon_sync_output(gx_device *dev)

{  
	PgFlush();
	return 0;
}

gx_color_index
photon_map_rgb_color(gx_device *dev, gx_color_value r, gx_color_value g,
  gx_color_value b)
{    

   return (PgRGB(r/256,g/256,b/256));
	
}    

/* I have yet to find a situation that calls this (wmb)*/
int
photon_map_color_rgb(gx_device *dev, gx_color_index color,
  gx_color_value prgb[3])
{		
	fprintf(stderr,"photon_map_color_rgb:\n");

	return 0;
}

int
photon_output_page(gx_device *dev, int num_copies, int flush)
{
	PgFlush();
	return 0;
}

